@@ -1,41 +1,57 @@
-.simulVasicekProcPhylTree<-function(phyltree,EvolModel,modelParams,EstimationParams=NULL,regimes=NULL,regimes.times=NULL,Simulparams=NULL,dropInternal=TRUE,bAllTrajectories=FALSE,M.error=NULL){
-## tree is assumed to be in ouch format
-## we don't need  descendent list as we have the tree in ouch format use @lineages
+## This file is part of mvSLOUCH
+
+## This software comes AS IS in the hope that it will be useful WITHOUT ANY WARRANTY, 
+## NOT even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+## Please understand that there may still be bugs and errors. Use it at your own risk. 
+## We take no responsibility for any errors or omissions in this package or for any misfortune 
+## that may befall you or others as a result of its use. Please send comments and report 
+## bugs to Krzysztof Bartoszek at krzbar@protonmail.ch .
+
+
+.simulVasicekProcPhylTree<-function(phyltree,EvolModel,modelParams,EstimationParams=NULL,regimes=NULL,regimes.times=NULL,Simulparams=NULL,dropInternal=TRUE,bAllTrajectories=FALSE,M.error=NULL,bKeep_tree=FALSE){
+## This is a dummy object that might become useful in the future when B for the mvSLOUCH model is estimated by GLS
+    lPrecalculates<-NULL 
 
-    if (bAllTrajectories && (is.null(Simulparams) || !is.element("step",names(Simulparams)))){
-	step<-min(c(0.00025,phyltree@depth/4000))
-	if (is.null(Simulparams)){Simulparams<-list(step=step)}
-	else{Simulparams$step<-step}
-    }
-## temporary simulation solution --- should consider a full model with this
     bJumpAtNode<-FALSE
     if (is.element("jump",names(Simulparams))){bJumpAtNode<-TRUE}
 ## -----------------------------------------------------------------------    
 
-    if (is.null(regimes.times)){regimes.times<-sapply(phyltree@epochs,rev,simplify=FALSE)}
-    if (is.null(regimes)){regimes<-sapply(regimes.times,function(reg){rep("reg.1",length(reg)-1)},simplify=FALSE)}
-    if (!is.list(regimes)){## The regimes are given as a vector in the ouch tree format --- Change them to a list
-        vregimes<-regimes
-        regimes<-sapply(phyltree@lineages[phyltree@term],function(epch,vregimes){
-            epch<-rev(epch)
-            as.character(sapply(epch[-1],function(reg,vregimes){vregimes[reg]},vregimes=vregimes,simplify=TRUE))
-	},vregimes=vregimes,simplify=FALSE)
-    }                                                                 
-    regimes.types.orig<-c()
-    for (i in 1: length(regimes)){regimes.types.orig<-c(regimes.types.orig,unique(regimes[[i]]))}
-    regimes.types.orig<-sort(unique(regimes.types.orig))
-    regimes.types<-1:length(regimes.types.orig)
-    regimes<-sapply(regimes,function(vregs,regimes.types.orig){
-        sapply(vregs,function(orgreg,regimes.types.orig){which(regimes.types.orig==orgreg)},regimes.types.orig=regimes.types.orig)
-    },regimes.types.orig=regimes.types.orig,simplify=FALSE)
-                                                        
+    if ((EvolModel=="bm")||(EvolModel=="bmStep")){
+	regimes<-NULL
+	regimes.times<-NULL
+    }
+    if ((EvolModel=="mvslouch") && .is_det0(modelParams$A,NULL)){
+	EvolModel<-"ouch"
+	modelParams<-.mvslouch_to_ouch_model(modelParams)
+	.my_warning("WARNING: A is a singular matrix, simulating under the OUOU model and not OUBM model so that distribution of output agrees with provided model parameters.",TRUE,FALSE)
+    }
+
+    regimesList<-.InitialRegimeSetup(phyltree=phyltree,regimes=regimes,regimes.times=regimes.times,mData=NULL,kX=NULL,kYX=NULL,root.regime=NULL,M.error=NULL)
+    bOKregimes<-regimesList$bOKregimes
+    if (!bOKregimes){.my_stop("The regimes, regimes.times and phyltree variables are not consistant. Cannot perform estimation. Please see manual on their format.",TRUE)}
+    regimes<-regimesList$regimes
+    regimes.times<-regimesList$regimes.times
+    regimes.types<-regimesList$regimes.types
+    root.regime<-regimesList$root.regime
+    regimes.types.orig<-regimesList$regimes.types.orig
+    phyltree<-regimesList$phyltree
+
+    if (bAllTrajectories && (is.null(Simulparams) || !is.element("step",names(Simulparams)))){
+	step<-min(c(0.001,phyltree$tree_height/2000))
+	if (is.null(Simulparams)){Simulparams<-list(step=step)}
+	else{Simulparams$step<-step}
+    }
     modelParams$regimes<-regimes
     modelParams$regimeTimes<-regimes.times
 
+    inumber_species<-phyltree$Ntips
+    inumber_nodes<-phyltree$Ntips+phyltree$Nnode
+    inumber_internal_nodes<-phyltree$Nnode
+    inumber_edges<-nrow(phyltree$edge)
     
      params<-list()
      params$EvolModel<-EvolModel
-     if (params$EvolModel=="slouch"){params$EvolModel<-"mvslouch";print("Call to slouch not yet implemented, using mvslouch model. You might need to run again with correct input structure.")}  
+     if (params$EvolModel=="slouch"){params$EvolModel<-"mvslouch";.my_warning("Call to slouch not yet implemented, using mvslouch model. You might need to run again with correct input structure. \n",TRUE,FALSE)}  
      if (!is.element("method",names(params))){params$method<-"glsgc"}
      if (params$EvolModel=="bm"){params$method<-"maxlik"}
      if (!is.element("bShouldPrint",names(params))){params$bShouldPrint<-TRUE}
@@ -45,7 +61,9 @@
      if (!is.element("minLogLik",names(params))){params$minLogLik<- -Inf}                                                           
 
     
-    lPrecalculates<-.calculate.Tree.dists(phyltree)
+    lPrecalculates<-list(vSpecies_times=NA)
+    lPrecalculates$vSpecies_times<-phyltree$time.of.nodes[phyltree$tip_species_index] 
+    
     if ((EvolModel=="bm")||(EvolModel=="bmStep")){
 	x0<-modelParams$vX0;kX<-nrow(modelParams$Sxx)
 	EstimationParams<-.set.estimparams(params,0,kX,length(regimes.types))
@@ -55,7 +73,6 @@
 	x0<-modelParams$vY0
 	EstimationParams<-.set.estimparams(params,kY,0,length(regimes.types))
         modelParams$precalcMatrices<-.decompEigenA.S(modelParams,lPrecalculates,NA,list(bCalcA=TRUE,bCovCalc=TRUE,dzetacalc=FALSE,lexptcalc=TRUE,kappacalc=FALSE,interceptcalc=FALSE),NULL)
-	##invlexpmtA<-sapply(modelParams$precalcMatrices[[3]]$lexpmtA,solve,simplify=FALSE)
     }
     if ((EvolModel=="mvslouch")||(EvolModel=="mvslouchStep")){
 	kY<-nrow(modelParams$A)
@@ -63,51 +80,56 @@
     	x0<-c(modelParams$vY0,modelParams$vX0)
     	EstimationParams<-.set.estimparams(params,kY,kX,length(regimes.types))
         modelParams$precalcMatrices<-.decompEigenA.S(modelParams,lPrecalculates,EstimationParams$designToEstim,list(bCalcA=TRUE,bCovCalc=TRUE,dzetacalc=FALSE,lexptcalc=TRUE,kappacalc=FALSE,interceptcalc=FALSE),NULL)
-	##invlexpmtA<-sapply(modelParams$precalcMatrices[[3]]$lexpmtA,solve,simplify=FALSE)
     }
-    RootNode<-phyltree@root
-    mTreeTraject<-matrix(NA,ncol=length(x0),nrow=length(phyltree@nodes))
-    rownames(mTreeTraject)<-phyltree@nodelabels
+    
+    RootNode<-phyltree$root_index
+
+    mTreeTraject<-matrix(NA,ncol=length(x0),nrow=inumber_nodes)
+    vnodelabels<-paste("node_",1:inumber_nodes,sep="")
+    vnodelabels[phyltree$tip_species_index]<-phyltree$tip.label
+    rownames(mTreeTraject)<-vnodelabels
     colnames(mTreeTraject)<-names(x0)
+    
     if (!is.na(x0[1])){ mTreeTraject[RootNode,]<-x0} ## the state of the root is x0    
     if (bJumpAtNode){vJumpNode<-rep(NA,nrow(mTreeTraject))}
     if (bAllTrajectories){
-    	if (is.element(EvolModel,c("bm","bmStep","ouch","ouchStep","mvslouch","mvslouchStep"))){ ## THIS NEEDS TO BE THOUGHT OUT IN CASE OF FUTURE DEVELOPMENT
-	    lFullTraject<-vector("list",length(phyltree@ancestors)-1)
-	    currbranch<-1
+    	if (is.element(EvolModel,c("bm","bmStep","ouch","ouchStep","mvslouch","mvslouchStep"))){
+	    ## we do the simulation on each edge
+	    lFullTraject<-vector("list",inumber_edges)
 	    if (!substr(EvolModel,start=nchar(EvolModel)-3,stop=nchar(EvolModel))=="Step"){
 	    	orgEvolModel<-EvolModel
 		EvolModel<-paste(EvolModel,"Step",sep="")
 	    }
 	}else{
-	    print("Cannot yet simulate full trajectory for the chosen model")
+	    .my_warning("Cannot yet simulate full trajectory for the chosen model",TRUE,TRUE)
 	    bAllTrajectories<-FALSE
 	}
     }else{lFullTraject<-NA}
 
-    for (Term in phyltree@term){## for each terminal node -- defines a lineage
-	vTermLineage<-rev(phyltree@lineages[[Term]]) ## we want to reverse it to start from root
+    for (Term in phyltree$tip_species_index){## for each terminal node -- defines a lineage
+	vTermLineage<-phyltree$path.from.root[[Term]]$nodes ## we want it to start from start of tree, but DO NOT need to reverse as due to setup of phyltree
 	i<-1
 	while(!is.na(mTreeTraject[vTermLineage[i],1])){i<-i+1}
 	if(i<length(vTermLineage)+1){## just to check if we haven't run over
 	    for (j in i:length(vTermLineage)){
+    		branch_index<-intersect(which(phyltree$edge[,1]==vTermLineage[j-1]),which(phyltree$edge[,2]==vTermLineage[j]))
+    		currbranch<-branch_index
     		if (bAllTrajectories){
-    		    lFullTraject[[currbranch]]<-vector("list",3)
-    		    names(lFullTraject[[currbranch]])<-c("branch","nodenames","trajectory")
+    		    lFullTraject[[currbranch]]<-vector("list",4)
+    		    names(lFullTraject[[currbranch]])<-c("branch","nodenames","trajectory","branch_index")
     		    lFullTraject[[currbranch]]$branch<-c(vTermLineage[j-1],vTermLineage[j])
-    		    lFullTraject[[currbranch]]$nodenames<-phyltree@nodelabels[c(vTermLineage[j-1],vTermLineage[j])]
+    		    lFullTraject[[currbranch]]$nodenames<-vnodelabels[c(vTermLineage[j-1],vTermLineage[j])]
+    		    lFullTraject[[currbranch]]$branch_index<-branch_index
     		}
     		Xprev<-mTreeTraject[vTermLineage[j-1],]
-		timeDiff<-phyltree@times[vTermLineage[j]]-phyltree@times[vTermLineage[j-1]]
+		timeDiff<-phyltree$edge.length[branch_index]
 
-## temporary simulation solution --- should consider a full model with this	
 		if (bJumpAtNode){		    
 		    bDoJump<-FALSE
 		    if (Simulparams$jump$jumptype=="ForBoth"){
                         if (is.na(vJumpNode[vTermLineage[j-1]])){
                             if (runif(1)<Simulparams$jump$jumpprob){bDoJump<-TRUE}
                             else{bDoJump<-FALSE;vJumpNode[vTermLineage[j-1]]<-0}
-                            #bDoJump<-TRUE
                         }
                     }
 		    if (Simulparams$jump$jumptype=="RandomLineage"){
@@ -121,7 +143,7 @@
 		    }
 		    if (bDoJump){		    	
 		    	## add according to distribution
-		    	if (Simulparams$jump$jumpdistrib=="Normal"){Xprev<-Xprev+rmvnorm(1,mean=Simulparams$jump$vMean,sigma=Simulparams$jump$mCov)}
+		    	if (Simulparams$jump$jumpdistrib=="Normal"){Xprev<-Xprev+mvtnorm::rmvnorm(1,mean=Simulparams$jump$vMean,sigma=Simulparams$jump$mCov)}
 			
 			## any post-add trajectory corrections
 			if (Simulparams$jump$jumptype=="ForBoth"){mTreeTraject[vTermLineage[j-1],]<-Xprev;vJumpNode[vTermLineage[j-1]]<-0}
@@ -137,40 +159,46 @@
 		if (EvolModel=="bm"){
 		    vMean<-Xprev[1:kX]
 		    mCov<-timeDiff*t(modelParams$Sxx)%*%modelParams$Sxx
-		    Xdrawn<-rmvnorm(n=1,mean=vMean,sigma=mCov) 
+		    Xdrawn<-mvtnorm::rmvnorm(n=1,mean=vMean,sigma=mCov) 
 		    Xdrawn<-Xdrawn[nrow(Xdrawn),]
 		}
 		if (EvolModel=="bmStep"){
-			itermNum<-Term-(phyltree@nnodes-phyltree@nterm)
+			## in principle (by phylo object standard) this should be Term
+			itermNum<-which(phyltree$tip_species_index==Term)
+			if (itermNum!=Term){.my_warning("WARNING: non-standard (for phylo object) numbering of tip species!",TRUE,FALSE)}
 	    		vX0<-Xprev[1:kX]
 	    		tmpmodelparams<-modelParams
 	    		tmpmodelparams$vX0<-matrix(vX0,ncol=1,nrow=kX)
 			Xdrawn<-.bm.simulate(step=ifelse(is.null(Simulparams),timeDiff,Simulparams$step),duration=timeDiff,modelParams=tmpmodelparams,regimes=NULL,regimes.times=NULL,mCov=NULL)
 			if (bAllTrajectories){
-			    Xdrawn[,1]<-Xdrawn[,1]+phyltree@times[vTermLineage[j-1]]
-			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn)]
+			    Xdrawn[,1]<-Xdrawn[,1]+phyltree$time.of.nodes[vTermLineage[j-1]]
+			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn),drop=FALSE]
 			}
 			Xdrawn<-Xdrawn[nrow(Xdrawn)-1,2:ncol(Xdrawn)]
 		}
 		if (EvolModel=="ouchStep"){
-			itermNum<-Term-(phyltree@nnodes-phyltree@nterm)
-			vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree@times[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree@times[vTermLineage[j]]))
+			## in principle (by phylo object standard) this should be Term
+			itermNum<-which(phyltree$tip_species_index==Term)
+			if (itermNum!=Term){.my_warning("WARNING: non-standard (for phylo object) numbering of tip species!",TRUE,FALSE)}
+			vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree$time.of.nodes[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree$time.of.nodes[vTermLineage[j]]))
 			regimesCurrTimes<-regimes.times[[itermNum]][vWhichTimes[1:(length(vWhichTimes))]]		
 			regimesCurr<-regimes[[itermNum]][vWhichTimes[1:(length(vWhichTimes)-1)]]		
 	    		vY0<-Xprev[1:kY]
 	    		tmpmodelparams<-modelParams
 	    		tmpmodelparams$vY0<-matrix(vY0,ncol=1,nrow=kY)
-	    		regimesCurrTimes<-regimesCurrTimes-phyltree@times[vTermLineage[j-1]]
+	    		regimesCurrTimes<-regimesCurrTimes-phyltree$time.of.nodes[vTermLineage[j-1]]
 			Xdrawn<-.ouch.simulate(step=ifelse(is.null(Simulparams),timeDiff,Simulparams$step),duration=timeDiff,modelParams=tmpmodelparams,regimes=regimesCurr,regimes.times=regimesCurrTimes,mCov=NULL)			
 			if (bAllTrajectories){
-			    Xdrawn[,1]<-Xdrawn[,1]+phyltree@times[vTermLineage[j-1]]
-			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn)]
+			    Xdrawn[,1]<-Xdrawn[,1]+phyltree$time.of.nodes[vTermLineage[j-1]]
+			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn),drop=FALSE]
 			}
 			Xdrawn<-Xdrawn[nrow(Xdrawn)-1,2:ncol(Xdrawn)]
 		}
 		if (EvolModel=="mvslouchStep"){
-			itermNum<-Term-(phyltree@nnodes-phyltree@nterm)
-			vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree@times[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree@times[vTermLineage[j]]))
+			## in principle (by phylo object standard) this should be Term
+			itermNum<-which(phyltree$tip_species_index==Term)
+			if (itermNum!=Term){.my_warning("WARNING: non-standard (for phylo object) numbering of tip species!",TRUE,FALSE)}
+			vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree$time.of.nodes[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree$time.of.nodes[vTermLineage[j]]))
 			regimesCurrTimes<-regimes.times[[itermNum]][vWhichTimes[1:(length(vWhichTimes))]]		
 			regimesCurr<-regimes[[itermNum]][vWhichTimes[1:(length(vWhichTimes)-1)]]		
 	    		vY0<-Xprev[1:kY]
@@ -178,76 +206,83 @@
 	    		tmpmodelparams<-modelParams
 	    		tmpmodelparams$vY0<-matrix(vY0,ncol=1,nrow=kY)
 	    		tmpmodelparams$vX0<-matrix(vX0,ncol=1,nrow=kX)
-	    		regimesCurrTimes<-regimesCurrTimes-phyltree@times[vTermLineage[j-1]]
+	    		regimesCurrTimes<-regimesCurrTimes-phyltree$time.of.nodes[vTermLineage[j-1]]
 			Xdrawn<-.mvslouch.simulate(step=ifelse(is.null(Simulparams),timeDiff,Simulparams$step),duration=timeDiff,modelParams=tmpmodelparams,regimes=regimesCurr,regimes.times=regimesCurrTimes,mCov=NULL)
 			if (bAllTrajectories){
-			    Xdrawn[,1]<-Xdrawn[,1]+phyltree@times[vTermLineage[j-1]]
-			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn)]
+			    Xdrawn[,1]<-Xdrawn[,1]+phyltree$time.of.nodes[vTermLineage[j-1]]
+			    lFullTraject[[currbranch]]$trajectory<-Xdrawn[1:(nrow(Xdrawn)-1),1:ncol(Xdrawn),drop=FALSE]
 			}
 			Xdrawn<-Xdrawn[nrow(Xdrawn)-1,2:ncol(Xdrawn)]
 		}
 
 		if ((EvolModel=="ouch")||(EvolModel=="mvslouch")){
-		    itermNum<-Term-(phyltree@nnodes-phyltree@nterm)
-		    vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree@times[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree@times[vTermLineage[j]]))
+		    ## in principle (by phylo object standard) this should be Term
+		    itermNum<-which(phyltree$tip_species_index==Term)
+		    if (itermNum!=Term){.my_warning("WARNING: non-standard (for phylo object) numbering of tip species!",TRUE,FALSE)}
+		    vWhichTimes<-intersect(which(regimes.times[[itermNum]]>=phyltree$time.of.nodes[vTermLineage[j-1]]),which(regimes.times[[itermNum]]<=phyltree$time.of.nodes[vTermLineage[j]]))
 	    	    vY0<-Xprev[1:kY]
 	    	    mPsi<-modelParams$mPsi
 	    	    mPsi0<-modelParams$mPsi0
-#	    	    expmtA<-modelParams$precalcMatrices[[3]]$lexpmtA[[itermNum]]
-#	    	    exptjA<-modelParams$precalcMatrices[[3]]$lexptjA[[itermNum]][vWhichTimes]
 
-		    expmtAcurr<-.calc.exptA(t=-timeDiff,modelParams$precalcMatrices[[1]])   ##A=modelParams$A) ## correct the mean value structure we are only moving along a single branch
-##		    exptAcorr<-expmtAcurr%*%.calc.exptA(t=-phyltree@times[vTermLineage[j-1]],A=modelParams$A)%*%invlexpmtA[[itermNum]]    ## and not through the whole tree, we want the mean at the branch end
-##		    exptjA<-sapply(modelParams$precalcMatrices[[3]]$lexptjA[[itermNum]][vWhichTimes],function(mexptjA,exptAcorr){exptAcorr%*%mexptjA},exptAcorr=exptAcorr,simplify=FALSE)
+		    expmtAcurr<-.calc.exptA(t=-timeDiff,modelParams$precalcMatrices[[1]])   
 
 		    exptjA<-sapply(c(regimes.times[[itermNum]][vWhichTimes]-regimes.times[[itermNum]][vWhichTimes[1]]-timeDiff),function(t,precalc){.calc.exptA(t=t,precalc)},precalc=modelParams$precalcMatrices[[1]],simplify=FALSE)		    
 	    	    regimesCurr<-regimes[[itermNum]][vWhichTimes[1:(length(vWhichTimes)-1)]]		
 	    	    names(regimesCurr)<-NULL
 		    if (EvolModel=="ouch"){
-#		        vMean<-.calc.mean.ouch.mv(expmtA,vY0,mPsi,mPsi0,exptjA,regimesCurr)
 		        vMean<-.calc.mean.ouch.mv(expmtAcurr,vY0,mPsi,mPsi0,exptjA,regimesCurr)
 		        mCov<-.calc.cov.ouch.mv(timeDiff,modelParams$precalcMatrices[[1]],modelParams$precalcMatrices[[2]])
 	    	    }		
 		    if (EvolModel=="mvslouch"){## we do not simulate the trajectory -> just draw from appropriate distribution
 			vX0<-Xprev[(kY+1):(length(Xprev))]
 			A1B<-modelParams$precalcMatrices[[1]]$A1B		
-#			vMean<-.calc.mean.slouch.mv(expmtA,A1B,vY0,vX0,mPsi,mPsi0,exptjA,regimesCurr)
     			vMean<-.calc.mean.slouch.mv(expmtAcurr,A1B,vY0,vX0,mPsi,mPsi0,exptjA,regimesCurr)
 			mCov<-.calc.cov.slouch.mv(timeDiff,modelParams$precalcMatrices[[1]],modelParams$precalcMatrices[[2]])		
 		    }
 		    
-		    Xdrawn<-rmvnorm(n=1,mean=vMean,sigma=mCov) 
+		    Xdrawn<-mvtnorm::rmvnorm(n=1,mean=vMean,sigma=mCov) 
 		    Xdrawn<-Xdrawn[nrow(Xdrawn),]
 		}
-		if(bAllTrajectories){currbranch<-currbranch+1}
 		mTreeTraject[vTermLineage[j],]<-Xdrawn
 	    }
-	}else{print("Something is wrong with tree --- seems to be a network")}	
+	}else{.my_stop("Something is wrong with tree---seems to be a network",TRUE)}	
     }    
-    if (dropInternal){mTreeTraject[-c(phyltree@term),]<-NA}
+
     
     if (EvolModel=="bm"){if (!is.null(colnames(modelParams$Sxx))){colnames(mTreeTraject)<-colnames(modelParams$Sxx)}}
     if (EvolModel=="ouch"){if (!is.null(colnames(modelParams$A))){colnames(mTreeTraject)<-colnames(modelParams$A)}}
     if (EvolModel=="mvslouch"){if ((!is.null(colnames(modelParams$A)))&&(!is.null(colnames(modelParams$Sxx)))){colnames(mTreeTraject)<-c(colnames(modelParams$A),colnames(modelParams$Sxx))}}
-    
+
     if (!is.null(M.error)){
-	M.error<-.createCovariancematrix(M.error,phyltree@nterm,ncol(mTreeTraject),NULL,"measurement error")
-    
-	m.errors<-rmvnorm(1,mean=rep(0,ncol(M.error)),sigma=M.error)
-	m.errors<-matrix(m.errors,nrow=phyltree@nterm,ncol=ncol(mTreeTraject),byrow=TRUE)
-	mTreeTraject[c(phyltree@term),]<-mTreeTraject[(phyltree@term),]+m.errors
+	M.error<-.createMeasurementError(M.error,phyltree$Ntips,ncol(mTreeTraject))
+    	for (i in phyltree$tip_species_index){
+    	    Term<-phyltree$tip_species_index[i]
+    	    mErrorCov<-matrix(NA,ncol(mTreeTraject),ncol(mTreeTraject))
+	    if (is.list(M.error)){mErrorCov<-M.error[[i]]}else{mErrorCov<-M.error}
+    	    m.errors<-mvtnorm::rmvnorm(1,mean=rep(0,ncol(mErrorCov)),sigma=mErrorCov)
+	    mTreeTraject[Term,]<-mTreeTraject[Term,]+m.errors
+	    if (bAllTrajectories){
+		branch_index<-which(phyltree$edge[,2]==Term)
+		lFullTraject[[branch_index]]$trajectory[nrow(lFullTraject[[branch_index]]$trajectory),2:ncol(lFullTraject[[branch_index]]$trajectory)]<-lFullTraject[[branch_index]]$trajectory[nrow(lFullTraject[[branch_index]]$trajectory),2:ncol(lFullTraject[[branch_index]]$trajectory)]+m.errors
+	    }
+	}
     }
-    mTreeTraject<-as.data.frame(mTreeTraject)
+
+    if (dropInternal){mTreeTraject<-mTreeTraject[phyltree$tip_species_index,,drop=FALSE]}
+
+    if (is.null(colnames(mTreeTraject))){colnames(mTreeTraject)<-paste("trait_",1:ncol(mTreeTraject),sep="")}
+    if (bAllTrajectories){
+	lFullTraject<-sapply(lFullTraject,function(ltraj,vtraitnames){colnames(ltraj$trajectory)<-c("time",vtraitnames);ltraj},vtraitnames=colnames(mTreeTraject),simplify=FALSE)}
     simulReturn<-vector("list",3)
     names(simulReturn)<-c("Tree","ExtantSample","FullTrajectory")
-    simulReturn$Tree<-phyltree
+    if (bKeep_tree){simulReturn$Tree<-phyltree}else{simulReturn$Tree<-NULL}
     simulReturn$ExtantSample<-mTreeTraject
     simulReturn$FullTrajectory<-lFullTraject
     simulReturn
 }
 
 
-drawPhylProcess<-function(PhylTraitProcess,phyltree=NULL,vColours="black",plotlayout=c(1,1),additionalfigs=FALSE,modelParams=NULL,EvolModel=NULL,xlimits=NULL,ylimits=NULL){
+drawPhylProcess<-function(PhylTraitProcess,vTraitsToPlot=NULL,vColours="black",plotlayout=c(1,1),additionalfigs=FALSE,modelParams=NULL,EvolModel=NULL,xlimits=NULL,ylimits=NULL){
     ## prepare data matrix to plot
     mData<-NA
     if (is.list(PhylTraitProcess)){
@@ -260,16 +295,20 @@ drawPhylProcess<-function(PhylTraitProcess,phyltree=NULL,vColours="black",plotla
     
     if (!is.na(mData[1])){    
 	ntraits<-ncol(mData)-1
+	if (is.null(vTraitsToPlot)){vTraitsToPlot<-2:(ntraits+1)}
+	if (!is.numeric(vTraitsToPlot)){.my_stop("The trait column numbers in vTraitsToPlot have to be numeric!",TRUE)}
+	if (length(setdiff(vTraitsToPlot,2:(ntraits+1)))>0){.my_stop(paste("Wrong trait column numbers provided in vTraitsToPlot object. The trait column numbers has to be a vector of integers in the range 2 to ",ntraits+1,".",sep=""),TRUE)}
+	if (length(vTraitsToPlot)>length(unique(vTraitsToPlot))){.my_warning("There are traits that will be plotted multiple times!",TRUE,FALSE)}
 	## check if we have the colors
 	if ((length(vColours)==0)||(is.na(vColours[1]))||(is.null(vColours))){vColours<-"black"}
 	if (length(vColours)<ntraits){vColours<-rep(vColours,length.out=ntraits)}
     
 	## prepare plot area
-	if (prod(plotlayout)<ntraits){plotlayout<-c(1,ntraits);print("WARNING : Possibly ugly plot layout, change parameter plotlayout!")}
+	if (prod(plotlayout)<ntraits){plotlayout<-c(1,length(vTraitsToPlot));.my_warning("WARNING : Possibly ugly plot layout, change parameter plotlayout! \n",TRUE,FALSE)}
     
 	## plot
 	par(mfrow=(plotlayout))
-	for (i in 2:(ntraits+1)){
+	for (i in vTraitsToPlot){
 	    minmaxx<-NA
 	    if (!is.null(xlimits)){
 		if ((is.vector(xlimits))&&(length(xlimits)==2)){minmaxx<-xlimits}
@@ -278,7 +317,7 @@ drawPhylProcess<-function(PhylTraitProcess,phyltree=NULL,vColours="black",plotla
 		if ((is.matrix(xlimits))&&(ncol(xlimits)==2)&&(nrow(xlimits)==1)){minmaxx<-xlimits[1,]}
 		if ((is.matrix(xlimits))&&(ncol(xlimits)==2)&&(nrow(xlimits)==ntraits)){minmaxx<-xlimits[i-1,]}		
 	    }
-	    if (is.na(minmaxx)){
+	    if (any(is.na(minmaxx))){
 		minmaxx<-c(min(mData[,i]),max(mData[,i]))
 	    }	    
 	    minmaxy<-NA
@@ -289,7 +328,7 @@ drawPhylProcess<-function(PhylTraitProcess,phyltree=NULL,vColours="black",plotla
 		if ((is.matrix(ylimits))&&(ncol(ylimits)==2)&&(nrow(ylimits)==1)){minmaxy<-ylimits[1,]}
 		if ((is.matrix(ylimits))&&(ncol(ylimits)==2)&&(nrow(ylimits)==ntraits)){minmaxy<-ylimits[i-1,]}		
 	    }
-	    if (is.na(minmaxy)){
+	    if (any(is.na(minmaxy))){
 	    	minmaxy<-c(min(mData[,1]),max(mData[,1]))
 	    }	    
 
@@ -304,7 +343,7 @@ drawPhylProcess<-function(PhylTraitProcess,phyltree=NULL,vColours="black",plotla
 		}
 	    }
 	}
-    }else{print("Error: wrong data provided to plotting function")}
+    }else{.my_stop("Error: wrong data provided to plotting function",TRUE)}
     NA        
 }
 
